package category;

import exeptions.CategoryAlreadyExists;
import exeptions.NonUniqueId;

import java.util.Optional;
import java.util.Set;

public class CategoryRepository {

	final private CategoryPersistenceAdapter categoryPersistenceAdapter;
	final private Set<Category> categories;

	public CategoryRepository(CategoryPersistenceAdapter categoryPersistenceAdapter) {
		this.categoryPersistenceAdapter = categoryPersistenceAdapter;
		this.categories = categoryPersistenceAdapter.getAll();
	}

	public Optional<Category> getByName(CategoryName name) {
		return categories.stream().filter(category -> category.getName().equals(name)).findFirst();
	}

	public Optional<Category> getById(CategoryId id) {
		return categories.stream().filter(category -> category.getId().equals(id)).findFirst();
	}

	public Set<Category> getAll(){
		return categories;
	}

	public void add(Category category) {
		checkDuplicates(category);

		categories.add(category);
		categoryPersistenceAdapter.add(category);
	}

	private void checkDuplicates(Category category) {
		if (this.getByName(category.getName()).isPresent()) {
			throw new CategoryAlreadyExists("A category with the name " + category.getName() + " already exits.");
		}

		if (this.getById(category.getId()).isPresent()) {
			throw new NonUniqueId("A category with the id " + category.getId() + " already exits.");
		}
	}
}
